@@ -2,7 +2,7 @@ import os
 import localstack_client.config
 
 # LocalStack version
-VERSION = '0.12.12'
+VERSION = '0.12.13.1'
 
 # constant to represent the "local" region, i.e., local machine
 REGION_LOCAL = 'local'
@@ -14,6 +14,7 @@ ENV_DEV = 'dev'
 HEADER_LOCALSTACK_EDGE_URL = 'x-localstack-edge'
 HEADER_LOCALSTACK_REQUEST_URL = 'x-localstack-request-url'
 HEADER_LOCALSTACK_TARGET = 'x-localstack-target'
+HEADER_AMZN_ERROR_TYPE = 'X-Amzn-Errortype'
 
 # backend service ports, for services that are behind a proxy (counting down from 4566)
 DEFAULT_PORT_EDGE = 4566
@@ -149,3 +150,6 @@ S3_STATIC_WEBSITE_HOSTNAME = 's3-website.%s' % LOCALHOST_HOSTNAME
 
 # port for debug py
 DEFAULT_DEVELOP_PORT = 5678
+
+# Default bucket name of the s3 bucket used for local lambda development
+DEFAULT_BUCKET_MARKER_LOCAL = '__local__'
