@@ -18,21 +18,21 @@ from localstack.services.kinesis import kinesis_listener
 from localstack.services.awslambda import lambda_api
 from localstack.services.apigateway import helpers
 from localstack.services.generic_proxy import ProxyListener
-from localstack.utils.aws.aws_responses import flask_to_requests_response, requests_response, LambdaResponse
+from localstack.utils.aws.aws_responses import (flask_to_requests_response, requests_response, LambdaResponse,
+    request_response_stream)
 from localstack.services.apigateway.helpers import (get_resource_for_path, handle_authorizers, handle_validators,
-    handle_accounts, extract_query_string_params, extract_path_params, make_error_response,
-    get_cors_response, hande_base_path_mappings)
+    handle_accounts, handle_vpc_links, extract_query_string_params, extract_path_params, make_error_response,
+    get_cors_response, handle_base_path_mappings, handle_client_certificates,
+    PATH_REGEX_AUTHORIZERS, PATH_REGEX_CLIENT_CERTS, PATH_REGEX_PATH_MAPPINGS, PATH_REGEX_RESPONSES,
+    PATH_REGEX_VALIDATORS)
 
 # set up logger
-LOGGER = logging.getLogger(__name__)
+LOG = logging.getLogger(__name__)
 
 # regex path patterns
-PATH_REGEX_AUTHORIZERS = r'^/restapis/([A-Za-z0-9_\-]+)/authorizers(\?.*)?'
-PATH_REGEX_VALIDATORS = r'^/restapis/([A-Za-z0-9_\-]+)/requestvalidators(\?.*)?'
-PATH_REGEX_RESPONSES = r'^/restapis/([A-Za-z0-9_\-]+)/gatewayresponses(/[A-Za-z0-9_\-]+)?(\?.*)?'
-PATH_REGEX_USER_REQUEST = r'^/restapis/([A-Za-z0-9_\-]+)/([A-Za-z0-9_\-]+)/%s/(.*)$' % PATH_USER_REQUEST
-PATH_REGEX_PATH_MAPPINGS = r'/domainnames/([^/]+)/basepathmappings(/.*)?'
 HOST_REGEX_EXECUTE_API = r'(.*://)?([a-zA-Z0-9-]+)\.execute-api\..*'
+TARGET_REGEX_S3_URI = r'^arn:aws:apigateway:[a-zA-Z0-9\-]+:s3:path/(?P<bucket>[^/]+)/(?P<object>.+)$'
+PATH_REGEX_USER_REQUEST = r'^/restapis/([A-Za-z0-9_\-]+)/([A-Za-z0-9_\-]+)/%s/(.*)$' % PATH_USER_REQUEST
 
 # Maps API IDs to list of gateway responses
 GATEWAY_RESPONSES = {}
@@ -45,7 +45,9 @@ class AuthorizationError(Exception):
 class ProxyListenerApiGateway(ProxyListener):
     def forward_request(self, method, path, data, headers):
         if re.match(PATH_REGEX_USER_REQUEST, path):
-            return invoke_rest_api_from_request(method, path, data, headers)
+            result = invoke_rest_api_from_request(method, path, data, headers)
+            if result is not None:
+                return result
 
         data = data and json.loads(to_str(data))
 
@@ -80,11 +82,16 @@ class ProxyListenerApiGateway(ProxyListener):
             result = None
             if path == '/account':
                 result = handle_accounts(method, path, data, headers)
-            if re.match(PATH_REGEX_PATH_MAPPINGS, path):
-                result = hande_base_path_mappings(method, path, data, headers)
+            elif path.startswith('/vpclinks'):
+                result = handle_vpc_links(method, path, data, headers)
+            elif re.match(PATH_REGEX_PATH_MAPPINGS, path):
+                result = handle_base_path_mappings(method, path, data, headers)
+            elif re.match(PATH_REGEX_CLIENT_CERTS, path):
+                result = handle_client_certificates(method, path, data, headers)
+
             if result is not None:
                 response.status_code = 200
-                aws_responses.set_response_content(response, result)
+                aws_responses.set_response_content(response, result, getattr(result, 'headers', {}))
 
         # publish event
         if method == 'POST' and path == '/restapis':
@@ -211,6 +218,19 @@ def update_content_length(response):
         response.headers['Content-Length'] = str(len(response.content))
 
 
+def apply_request_parameter(integration, path_params):
+    request_parameters = integration.get('requestParameters', None)
+    uri = integration.get('uri') or integration.get('integrationUri') or ''
+    if request_parameters:
+        for key in path_params:
+            # check if path_params is present in the integration request parameters
+            request_param_key = f'integration.request.path.{key}'
+            request_param_value = f'method.request.path.{key}'
+            if request_parameters.get(request_param_key, None) == request_param_value:
+                uri = uri.replace(f'{{{key}}}', path_params[key])
+    return uri
+
+
 def apply_template(integration, req_res_type, data, path_params={}, query_params={}, headers={}):
     integration_type = integration.get('type') or integration.get('integrationType')
     if integration_type in ['HTTP', 'AWS']:
@@ -235,6 +255,26 @@ def apply_template(integration, req_res_type, data, path_params={}, query_params
     return data
 
 
+def apply_response_parameters(response, integration, api_id=None):
+    int_responses = integration.get('integrationResponses') or {}
+    if not int_responses:
+        return response
+    entries = list(int_responses.keys())
+    return_code = str(response.status_code)
+    if return_code not in entries:
+        if len(entries) > 1:
+            LOG.info('Found multiple integration response status codes: %s' % entries)
+            return response
+        return_code = entries[0]
+    response_params = int_responses[return_code].get('responseParameters', {})
+    for key, value in response_params.items():
+        # TODO: add support for method.response.body, etc ...
+        if str(key).lower().startswith('method.response.header.'):
+            header_name = key[len('method.response.header.'):]
+            response.headers[header_name] = value.strip("'")
+    return response
+
+
 def get_api_id_stage_invocation_path(path, headers):
     path_match = re.search(PATH_REGEX_USER_REQUEST, path)
     host_header = headers.get('Host', '')
@@ -298,23 +338,36 @@ def invoke_rest_api(api_id, stage, method, invocation_path, data, headers, path=
 
 def invoke_rest_api_integration(api_id, stage, integration, method, path, invocation_path, data, headers,
         resource_path, context={}, resource_id=None, response_templates={}):
+    response = None
+    try:
+        response = invoke_rest_api_integration_backend(
+            api_id, stage, integration, method, path, invocation_path,
+            data, headers, resource_path, context=context, resource_id=resource_id,
+            response_templates=response_templates)
+        response = apply_response_parameters(response, integration, api_id=api_id)
+        return response
+    except Exception as e:
+        LOG.warning(str(e))
+        return make_error_response(str(e), 400)
+
+
+def invoke_rest_api_integration_backend(api_id, stage, integration, method, path, invocation_path,
+        data, headers, resource_path, context={}, resource_id=None, response_templates={}):
 
     relative_path, query_string_params = extract_query_string_params(path=invocation_path)
     integration_type_orig = integration.get('type') or integration.get('integrationType') or ''
     integration_type = integration_type_orig.upper()
     uri = integration.get('uri') or integration.get('integrationUri') or ''
-
+    try:
+        path_params = extract_path_params(path=relative_path, extracted_path=resource_path)
+    except Exception:
+        path_params = {}
     if (uri.startswith('arn:aws:apigateway:') and ':lambda:path' in uri) or uri.startswith('arn:aws:lambda'):
         if integration_type in ['AWS', 'AWS_PROXY']:
             func_arn = uri
             if ':lambda:path' in uri:
                 func_arn = uri.split(':lambda:path')[1].split('functions/')[1].split('/invocations')[0]
 
-            try:
-                path_params = extract_path_params(path=relative_path, extracted_path=resource_path)
-            except Exception:
-                path_params = {}
-
             # apply custom request template
             data_str = data
             try:
@@ -364,10 +417,8 @@ def invoke_rest_api_integration(api_id, stage, integration, method, path, invoca
 
             return response
 
-        msg = 'API Gateway %s integration action "%s", method "%s" not yet implemented' % (
-            integration_type, uri, method)
-        LOGGER.warning(msg)
-        return make_error_response(msg, 404)
+        raise Exception('API Gateway %s integration action "%s", method "%s" not yet implemented' % (
+            integration_type, uri, method))
 
     elif integration_type == 'AWS':
         if 'kinesis:action/' in uri:
@@ -412,6 +463,34 @@ def invoke_rest_api_integration(api_id, stage, integration, method, path, invoca
             )
             response.headers['content-type'] = APPLICATION_JSON
             return response
+        elif 's3:path/' in uri and method == 'GET':
+            s3 = aws_stack.connect_to_service('s3')
+            uri_match = re.match(TARGET_REGEX_S3_URI, uri)
+            if uri_match:
+                bucket, object_key = uri_match.group('bucket', 'object')
+                LOG.debug('Getting request for bucket %s object %s', bucket, object_key)
+                try:
+                    object = s3.get_object(Bucket=bucket, Key=object_key)
+                except s3.exceptions.NoSuchKey:
+                    msg = 'Object %s not found' % object_key
+                    LOG.debug(msg)
+                    return make_error_response(msg, 404)
+
+                headers = aws_stack.mock_aws_request_headers(service='s3')
+
+                if object.get('ContentType'):
+                    headers['Content-Type'] = object['ContentType']
+
+                # stream used so large files do not fill memory
+                response = request_response_stream(
+                    stream=object['Body'],
+                    headers=headers
+                )
+                return response
+            else:
+                msg = 'Request URI does not match s3 specifications'
+                LOG.warning(msg)
+                return make_error_response(msg, 400)
 
         if method == 'POST':
             if uri.startswith('arn:aws:apigateway:') and ':sqs:path' in uri:
@@ -426,9 +505,8 @@ def invoke_rest_api_integration(api_id, stage, integration, method, path, invoca
                 result = common.make_http_request(url, method='POST', headers=headers, data=new_request)
                 return result
 
-        msg = 'API Gateway AWS integration action URI "%s", method "%s" not yet implemented' % (uri, method)
-        LOGGER.warning(msg)
-        return make_error_response(msg, 404)
+        raise Exception('API Gateway AWS integration action URI "%s", method "%s" not yet implemented' %
+            (uri, method))
 
     elif integration_type == 'AWS_PROXY':
         if uri.startswith('arn:aws:apigateway:') and ':dynamodb:action' in uri:
@@ -437,10 +515,11 @@ def invoke_rest_api_integration(api_id, stage, integration, method, path, invoca
             action = uri.split(':dynamodb:action')[1].split('&Table=')[0]
 
             if 'PutItem' in action and method == 'PUT':
-                response_template = response_templates.get('application/json', None)
+                response_template = response_templates.get('application/json')
 
                 if response_template is None:
                     msg = 'Invalid response template defined in integration response.'
+                    LOG.info('%s Existing: %s' % (msg, response_templates))
                     return make_error_response(msg, 404)
 
                 response_template = json.loads(response_template)
@@ -460,9 +539,8 @@ def invoke_rest_api_integration(api_id, stage, integration, method, path, invoca
                 response = requests_response(event_data, headers=aws_stack.mock_aws_request_headers())
                 return response
         else:
-            msg = 'API Gateway action uri "%s", integration type %s not yet implemented' % (uri, integration_type)
-            LOGGER.warning(msg)
-            return make_error_response(msg, 404)
+            raise Exception('API Gateway action uri "%s", integration type %s not yet implemented' %
+                (uri, integration_type))
 
     elif integration_type in ['HTTP_PROXY', 'HTTP']:
 
@@ -479,27 +557,23 @@ def invoke_rest_api_integration(api_id, stage, integration, method, path, invoca
         data = apply_template(integration, 'request', data)
         if isinstance(data, dict):
             data = json.dumps(data)
-
+        uri = apply_request_parameter(integration=integration, path_params=path_params)
         function = getattr(requests, method.lower())
         result = function(uri, data=data, headers=headers)
-
         # apply custom response template
         data = apply_template(integration, 'response', data)
-
         return result
 
     elif integration_type == 'MOCK':
-        # TODO: add logic for MOCK responses
-        pass
+        # return empty response - details filled in via requestParameters above...
+        return requests_response({})
 
     if method == 'OPTIONS':
         # fall back to returning CORS headers if this is an OPTIONS request
         return get_cors_response(headers)
 
-    msg = ('API Gateway integration type "%s", method "%s", URI "%s" not yet implemented' %
+    raise Exception('API Gateway integration type "%s", method "%s", URI "%s" not yet implemented' %
            (integration_type, method, uri))
-    LOGGER.warning(msg)
-    return make_error_response(msg, 404)
 
 
 def get_stage_variables(api_id, stage):
