@@ -19,8 +19,8 @@ from localstack import config
 from localstack.utils import bootstrap
 from localstack.utils.aws import aws_stack
 from localstack.utils.common import (
-    CaptureOutput, FuncThread, TMP_FILES, short_uid, save_file, rm_rf, in_docker, long_uid,
-    now, to_str, to_bytes, run, cp_r, json_safe, get_free_tcp_port, rm_docker_container)
+    CaptureOutput, FuncThread, TMP_FILES, short_uid, save_file, rm_rf, in_docker, last_index_of,
+    long_uid, now, to_str, to_bytes, run, cp_r, json_safe, get_free_tcp_port, rm_docker_container)
 from localstack.services.install import INSTALL_PATH_LOCALSTACK_FAT_JAR
 from localstack.utils.aws.dead_letter_queue import lambda_error_to_dead_letter_queue
 from localstack.utils.aws.dead_letter_queue import sqs_error_to_dead_letter_queue
@@ -40,6 +40,10 @@ LAMBDA_SERVER_PORT_OFFSET = 5000
 LAMBDA_API_UNIQUE_PORTS = 500
 LAMBDA_API_PORT_OFFSET = 9000
 
+MAX_ENV_ARGS_LENGTH = 20000
+
+INTERNAL_LOG_PREFIX = 'ls-daemon: '
+
 # logger
 LOG = logging.getLogger(__name__)
 
@@ -188,28 +192,61 @@ class LambdaExecutor(object):
     def cleanup(self, arn=None):
         pass
 
-    def run_lambda_executor(self, cmd, event=None, func_details=None, env_vars={}):
-        kwargs = {'stdin': True, 'inherit_env': True, 'asynchronous': True}
-        env_vars = env_vars or {}
+    def run_lambda_executor(self, cmd, event=None, func_details=None, env_vars=None):
+        env_vars = dict(env_vars or {})
         runtime = func_details.runtime or ''
 
+        stdin_str = None
+        event_body = event if event is not None else env_vars.get('AWS_LAMBDA_EVENT_BODY')
+        event_body = json.dumps(event_body) if isinstance(event_body, dict) else event_body
+        event_body = event_body or ''
+        is_large_event = len(event_body) > MAX_ENV_ARGS_LENGTH
+
         is_provided = runtime.startswith(LAMBDA_RUNTIME_PROVIDED)
-        if func_details and is_provided and env_vars.get('DOCKER_LAMBDA_USE_STDIN') == '1':
-            # Note: certain "provided" runtimes (e.g., Rust programs) can block when we pass in
+        if not is_large_event and func_details and is_provided and env_vars.get('DOCKER_LAMBDA_USE_STDIN') == '1':
+            # Note: certain "provided" runtimes (e.g., Rust programs) can block if we pass in
             # the event payload via stdin, hence we rewrite the command to "echo ... | ..." below
             env_updates = {
                 'PATH': env_vars.get('PATH') or os.environ.get('PATH', ''),
-                'AWS_LAMBDA_EVENT_BODY': to_str(event),
+                'AWS_LAMBDA_EVENT_BODY': to_str(event_body),  # Note: seems to be needed for provided runtimes!
                 'DOCKER_LAMBDA_USE_STDIN': '1'
             }
             env_vars.update(env_updates)
             # Note: $AWS_LAMBDA_COGNITO_IDENTITY='{}' causes Rust Lambdas to hang
             env_vars.pop('AWS_LAMBDA_COGNITO_IDENTITY', None)
-            event = None
-            cmd = re.sub(r'(.*)(%s\s+(run|start))' % self._docker_cmd(), r'\1echo $AWS_LAMBDA_EVENT_BODY | \2', cmd)
+            cmd = re.sub(r'(.*)(%s\s+(run|start|exec))' % self._docker_cmd(),
+                r'\1echo $AWS_LAMBDA_EVENT_BODY | \2', cmd)
 
+        if is_large_event:
+            # in case of very large event payloads, we need to pass them via stdin
+            LOG.debug('Received large Lambda event payload (length %s) - passing via stdin' % len(event_body))
+            env_vars['DOCKER_LAMBDA_USE_STDIN'] = '1'
+
+        def add_env_var(cmd, name, value=None):
+            value = value or '$%s' % name
+            return re.sub(r'(%s)\s+(run|exec)\s+' % config.DOCKER_CMD,
+                r'\1 \2 -e %s="%s" ' % (name, value), cmd)
+
+        def rm_env_var(cmd, name):
+            return re.sub(r'-e\s+%s="?[^"\s]+"?' % name, '', cmd)
+
+        if env_vars.get('DOCKER_LAMBDA_USE_STDIN') == '1':
+            stdin_str = event_body
+            if not is_provided:
+                env_vars.pop('AWS_LAMBDA_EVENT_BODY', None)
+            if 'DOCKER_LAMBDA_USE_STDIN' not in cmd:
+                cmd = add_env_var(cmd, 'DOCKER_LAMBDA_USE_STDIN', '1')
+                cmd = rm_env_var(cmd, 'AWS_LAMBDA_EVENT_BODY')
+        else:
+            if 'AWS_LAMBDA_EVENT_BODY' not in env_vars:
+                env_vars['AWS_LAMBDA_EVENT_BODY'] = to_str(event_body)
+            cmd = add_env_var(cmd, 'AWS_LAMBDA_EVENT_BODY')
+            cmd = rm_env_var(cmd, 'DOCKER_LAMBDA_USE_STDIN')
+
+        kwargs = {'stdin': True, 'inherit_env': True, 'asynchronous': True}
         process = run(cmd, env_vars=env_vars, stderr=subprocess.PIPE, outfile=subprocess.PIPE, **kwargs)
-        result, log_output = process.communicate(input=event)
+        event_stdin_bytes = stdin_str and to_bytes(stdin_str)
+        result, log_output = process.communicate(input=event_stdin_bytes)
         try:
             result = to_str(result).strip()
         except Exception:
@@ -220,8 +257,12 @@ class LambdaExecutor(object):
         # will be part of the "result" variable here. Hence, make sure that we extract
         # only the *last* line of "result" and consider anything above that as log output.
         if isinstance(result, six.string_types) and '\n' in result:
-            additional_logs, _, result = result.rpartition('\n')
-            log_output += '\n%s' % additional_logs
+            lines = result.split('\n')
+            idx = last_index_of(lines, lambda line: line and not line.startswith(INTERNAL_LOG_PREFIX))
+            if idx >= 0:
+                result = lines[idx]
+                additional_logs = '\n'.join(lines[:idx] + lines[idx + 1:])
+                log_output += '\n%s' % additional_logs
 
         log_formatted = log_output.strip().replace('\n', '\n> ')
         func_arn = func_details and func_details.arn()
@@ -259,7 +300,7 @@ class LambdaExecutorContainers(LambdaExecutor):
         """ Return the event as a stdin string. """
         # amend the environment variables for execution
         environment['AWS_LAMBDA_EVENT_BODY'] = event_body
-        return None
+        return event_body.encode()
 
     def _execute(self, func_arn, func_details, event, context=None, version=None):
         lambda_cwd = func_details.cwd
@@ -273,7 +314,7 @@ class LambdaExecutorContainers(LambdaExecutor):
 
         # prepare event body
         if not event:
-            LOG.warning('Empty event body specified for invocation of Lambda "%s"' % func_arn)
+            LOG.info('Empty event body specified for invocation of Lambda "%s"' % func_arn)
             event = {}
         event_body = json.dumps(json_safe(event))
         stdin = self.prepare_event(environment, event_body)
@@ -331,7 +372,7 @@ class LambdaExecutorContainers(LambdaExecutor):
 
         # run Lambda executor and fetch invocation result
         LOG.info('Running lambda cmd: %s' % cmd)
-        result = self.run_lambda_executor(cmd, stdin, env_vars=environment, func_details=func_details)
+        result = self.run_lambda_executor(cmd, event=stdin, env_vars=environment, func_details=func_details)
 
         # clean up events file
         events_file_path and os.path.exists(events_file_path) and rm_rf(events_file_path)
@@ -371,16 +412,14 @@ class LambdaExecutorReuseContainers(LambdaExecutorContainers):
 
         # create/verify the docker container is running.
         LOG.debug('Priming docker container with runtime "%s" and arn "%s".', runtime, func_arn)
-        container_info = self.prime_docker_container(func_details, env_vars.items(), lambda_cwd)
-
-        # Note: currently "docker exec" does not support --env-file, i.e., environment variables can only be
-        # passed directly on the command line, using "-e" below. TODO: Update this code once --env-file is
-        # available for docker exec, to better support very large Lambda events (very long environment values)
-        exec_env_vars = ' '.join(['-e {}="${}"'.format(k, k) for (k, v) in env_vars.items()])
+        container_info = self.prime_docker_container(func_details, env_vars, lambda_cwd)
 
         if not command:
             command = '%s %s' % (container_info.entry_point, handler)
 
+        # create file with environment variables
+        env_vars_flag = Util.create_env_vars_file_flag(env_vars)
+
         # determine files to be copied into the container
         copy_command = ''
         docker_cmd = self._docker_cmd()
@@ -391,11 +430,11 @@ class LambdaExecutorReuseContainers(LambdaExecutorContainers):
 
         cmd = (
             '%s'
-            ' %s exec'
-            ' %s'  # env variables
+            ' %s exec -i'
+            ' %s'  # env variables file
             ' %s'  # container name
             ' %s'  # run cmd
-        ) % (copy_command, docker_cmd, exec_env_vars, container_info.name, command)
+        ) % (copy_command, docker_cmd, env_vars_flag, container_info.name, command)
         LOG.debug('Command for docker-reuse Lambda executor: %s' % cmd)
 
         return cmd
@@ -436,7 +475,7 @@ class LambdaExecutorReuseContainers(LambdaExecutorContainers):
             docker_cmd = self._docker_cmd()
 
             status = self.get_docker_container_status(func_arn)
-            LOG.debug('Priming docker container (status "%s"): %s' % (status, container_name))
+            LOG.debug('Priming Docker container (status "%s"): %s' % (status, container_name))
 
             docker_image = Util.docker_image_for_lambda(func_details)
 
@@ -476,7 +515,11 @@ class LambdaExecutorReuseContainers(LambdaExecutorContainers):
         docker_cmd = self._docker_cmd()
         container_name = self.get_container_name(func_details.arn())
 
-        env_vars_str = ' '.join(['-e {}={}'.format(k, cmd_quote(v)) for (k, v) in env_vars])
+        # make sure AWS_LAMBDA_EVENT_BODY is not set (otherwise causes issues with "docker exec ..." above)
+        env_vars.pop('AWS_LAMBDA_EVENT_BODY', None)
+
+        # create environment variables flag (either passed directly, or as env var file)
+        env_vars_flags = Util.create_env_vars_file_flag(env_vars)
 
         network = config.LAMBDA_DOCKER_NETWORK
         network_str = '--network="%s"' % network if network else ''
@@ -500,7 +543,6 @@ class LambdaExecutorReuseContainers(LambdaExecutorContainers):
             ' --entrypoint /bin/bash'  # Load bash when it starts.
             ' %s'
             ' --interactive'  # Keeps the container running bash.
-            ' -e AWS_LAMBDA_EVENT_BODY="$AWS_LAMBDA_EVENT_BODY"'
             ' -e HOSTNAME="$HOSTNAME"'
             ' -e LOCALSTACK_HOSTNAME="$LOCALSTACK_HOSTNAME"'
             ' -e EDGE_PORT="$EDGE_PORT"'
@@ -510,7 +552,7 @@ class LambdaExecutorReuseContainers(LambdaExecutorContainers):
             ' %s'  # additional flags
             ' %s'
         ) % (docker_cmd, rm_flag, container_name, mount_volume_str,
-            env_vars_str, network_str, dns_str, additional_flags, docker_image)
+            env_vars_flags, network_str, dns_str, additional_flags, docker_image)
         return cmd
 
     def get_container_entrypoint(self, docker_image):
@@ -726,14 +768,14 @@ class LambdaExecutorSeparateContainers(LambdaExecutorContainers):
         dns = config.LAMBDA_DOCKER_DNS
         dns_str = '--dns="%s"' % dns if dns else ''
 
-        env_vars_string = ' '.join(['-e {}="${}"'.format(k, k) for (k, v) in env_vars.items()])
+        env_vars_flag = Util.create_env_vars_file_flag(env_vars)
         debug_docker_java_port = '-p {p}:{p}'.format(p=Util.debug_java_port) if Util.debug_java_port else ''
         docker_cmd = self._docker_cmd()
         docker_image = Util.docker_image_for_lambda(func_details)
         rm_flag = Util.get_docker_remove_flag()
 
         # construct common flags for commands below
-        common_flags = ' '.join([env_vars_string, network_str, dns_str, additional_flags, rm_flag])
+        common_flags = ' '.join([env_vars_flag, network_str, dns_str, additional_flags, rm_flag])
 
         if config.LAMBDA_REMOTE_DOCKER:
             cp_cmd = ('%s cp "%s/." "$CONTAINER_ID:%s";' % (
@@ -877,8 +919,8 @@ class Util:
         docker_tag = runtime
         docker_image = config.LAMBDA_CONTAINER_REGISTRY
         # TODO: remove prefix once execution issues are fixed with dotnetcore/python lambdas
-        # See https://github.com/lambci/docker-lambda/pull/218
-        lambdas_to_add_prefix = ['dotnetcore2.0', 'dotnetcore2.1', 'python2.7', 'python3.6', 'python3.7']
+        #  See https://github.com/lambci/docker-lambda/pull/218
+        lambdas_to_add_prefix = ['dotnetcore2.0', 'dotnetcore2.1', 'python3.6', 'python3.7']
         if docker_image == 'lambci/lambda' and any(img in docker_tag for img in lambdas_to_add_prefix):
             docker_tag = '20191117-%s' % docker_tag
         if runtime == 'nodejs14.x':
@@ -886,8 +928,8 @@ class Util:
             docker_image = 'localstack/lambda-js'
         return '"%s:%s"' % (docker_image, docker_tag)
 
-    @classmethod
-    def get_docker_remove_flag(cls):
+    @staticmethod
+    def get_docker_remove_flag():
         return '--rm' if config.LAMBDA_REMOVE_CONTAINERS else ''
 
     @classmethod
@@ -917,6 +959,47 @@ class Util:
         result = ':'.join(entries)
         return result
 
+    @classmethod
+    def create_env_vars_file_flag(cls, env_vars, use_env_variable_names=True):
+        if not env_vars:
+            return ''
+        result = ''
+        env_vars = dict(env_vars)
+        if len(str(env_vars)) > MAX_ENV_ARGS_LENGTH:
+            # default ARG_MAX=131072 in Docker - let's create an env var file if the string becomes too long...
+            env_file = cls.mountable_tmp_file()
+            env_content = ''
+            for name, value in dict(env_vars).items():
+                if len(value) > MAX_ENV_ARGS_LENGTH:
+                    # each line in the env file has a max size as well (error "bufio.Scanner: token too long")
+                    continue
+                env_vars.pop(name)
+                value = value.replace('\n', '\\')
+                env_content += '%s=%s\n' % (name, value)
+            save_file(env_file, env_content)
+            result += '--env-file %s ' % env_file
+
+        if use_env_variable_names:
+            env_vars_str = ' '.join(['-e {k}="${k}"'.format(k=k) for k in env_vars.keys()])
+        else:
+            # TODO: we should remove this mid-term - shouldn't be using cmd_quote directly
+            env_vars_str = ' '.join(['-e {}={}'.format(k, cmd_quote(v)) for k, v in env_vars.items()])
+        result += env_vars_str
+        return result
+
+    @staticmethod
+    def rm_env_vars_file(env_vars_file_flag):
+        if not env_vars_file_flag or '--env-file' not in env_vars_file_flag:
+            return
+        env_vars_file = env_vars_file_flag.replace('--env-file', '').strip()
+        return rm_rf(env_vars_file)
+
+    @staticmethod
+    def mountable_tmp_file():
+        f = os.path.join(config.TMP_FOLDER, short_uid())
+        TMP_FILES.append(f)
+        return f
+
 
 # --------------
 # GLOBAL STATE
