@@ -13,7 +13,7 @@ from localstack.utils import common
 from localstack.utils.aws import aws_stack
 from localstack.utils.common import (
     camel_to_snake_case, select_attributes, canonical_json, md5, is_base64,
-    new_tmp_dir, save_file, rm_rf, mkdir, cp_r, short_uid)
+    new_tmp_dir, save_file, rm_rf, mkdir, cp_r, short_uid, keys_to_lower)
 from localstack.utils.testutil import create_zip_file
 from localstack.services.awslambda.lambda_api import (
     get_handler_file_from_name, LAMBDA_POLICY_NAME_PATTERN)
@@ -1032,6 +1032,7 @@ class GatewayMethod(GenericBaseModel):
                 kwargs['integrationHttpMethod'] = integration.get('IntegrationHttpMethod')
             if integration.get('Uri'):
                 kwargs['uri'] = integration.get('Uri')
+            kwargs['requestParameters'] = integration.get('RequestParameters') or {}
             return client.put_integration(**kwargs)
         kwargs['authorizationType'] = props.get('AuthorizationType')
 
@@ -1060,6 +1061,23 @@ class GatewayStage(GenericBaseModel):
     def get_physical_resource_id(self, attribute=None, **kwargs):
         return self.props.get('id')
 
+    @staticmethod
+    def get_deploy_templates():
+        def get_params(params, **kwargs):
+            result = keys_to_lower(params)
+            param_names = ['restApiId', 'stageName', 'deploymentId', 'description',
+                'cacheClusterEnabled', 'cacheClusterSize', 'variables', 'documentationVersion',
+                'canarySettings', 'tracingEnabled', 'tags']
+            result = select_attributes(result, param_names)
+            result['tags'] = {t['key']: t['value'] for t in result.get('tags', [])}
+            return result
+        return {
+            'create': {
+                'function': 'create_stage',
+                'parameters': get_params
+            }
+        }
+
 
 class GatewayUsagePlan(GenericBaseModel):
     @staticmethod
@@ -2143,3 +2161,41 @@ class EC2VPCGatewayAttachment(GenericBaseModel):
                 'function': _attach_gateway
             }
         }
+
+
+class ResourceGroupsGroup(GenericBaseModel):
+    @staticmethod
+    def cloudformation_type():
+        return 'AWS::ResourceGroups::Group'
+
+    def fetch_state(self, stack_name, resources):
+        client = aws_stack.connect_to_service('resource-groups')
+        result = client.list_groups().get('Groups', [])
+        result = [g for g in result if g['Name'] == self.props['Name']]
+        return (result or [None])[0]
+
+    def get_physical_resource_id(self, attribute=None, **kwargs):
+        if attribute in REF_ARN_ATTRS:
+            return self.props.get('GroupArn')
+        return self.props.get('Name')
+
+    @classmethod
+    def get_deploy_templates(cls):
+        return {
+            'create': {
+                'function': 'create_group',
+                'parameters': {
+                    'Name': 'Name',
+                    'Description': 'Description',
+                    'ResourceQuery': 'ResourceQuery',
+                    'Configuration': 'Configuration',
+                    'Tags': params_list_to_dict('Tags')
+                }
+            },
+            'delete': {
+                'function': 'delete_group',
+                'parameters': {
+                    'Group': 'Name'
+                }
+            }
+        }
