@@ -1,24 +1,28 @@
 #!/usr/bin/env python
-
 import re
 import os
 import sys
 import glob
+import platform
 import time
 import shutil
 import logging
 import tempfile
+
+import requests
+
 from localstack import config
+from localstack.config import KINESIS_PROVIDER
 from localstack.constants import MODULE_MAIN_PATH, INSTALL_DIR_INFRA
 from localstack.utils.common import is_windows
 from localstack.utils import bootstrap
 from localstack.constants import (DEFAULT_SERVICE_PORTS, ELASTICMQ_JAR_URL, STS_JAR_URL,
     ELASTICSEARCH_URLS, ELASTICSEARCH_DEFAULT_VERSION, ELASTICSEARCH_PLUGIN_LIST,
     ELASTICSEARCH_DELETE_MODULES, DYNAMODB_JAR_URL, DYNAMODB_JAR_URL_ALPINE, LOCALSTACK_MAVEN_VERSION,
-    STEPFUNCTIONS_ZIP_URL, KMS_URL_PATTERN, LOCALSTACK_INFRA_PROCESS)
+    KMS_URL_PATTERN, LOCALSTACK_INFRA_PROCESS)
 if __name__ == '__main__':
     bootstrap.bootstrap_installation()
-# flake8: noqa: E402
+# noqa: E402
 from localstack.utils.common import (
     download, parallelize, run, mkdir, load_file, save_file, unzip, untar, rm_rf,
     chmod_r, is_alpine, in_docker, get_arch, new_tmp_file)
@@ -36,6 +40,7 @@ INSTALL_PATH_STEPFUNCTIONS_JAR = os.path.join(INSTALL_DIR_STEPFUNCTIONS, 'StepFu
 INSTALL_PATH_KMS_BINARY_PATTERN = os.path.join(INSTALL_DIR_KMS, 'local-kms.<arch>.bin')
 INSTALL_PATH_ELASTICMQ_JAR = os.path.join(INSTALL_DIR_ELASTICMQ, 'elasticmq-server.jar')
 INSTALL_PATH_KINESALITE_CLI = os.path.join(INSTALL_DIR_NPM, 'kinesalite', 'cli.js')
+INSTALL_PATH_KINESIS_MOCK = os.path.join(INSTALL_DIR_INFRA, 'kinesis-mock')
 URL_LOCALSTACK_FAT_JAR = ('https://repo1.maven.org/maven2/' +
     'cloud/localstack/localstack-utils/{v}/localstack-utils-{v}-fat.jar').format(v=LOCALSTACK_MAVEN_VERSION)
 MARKER_FILE_LIGHT_VERSION = '%s/.light-version' % INSTALL_DIR_INFRA
@@ -44,8 +49,12 @@ ARTIFACTS_REPO = 'https://github.com/localstack/localstack-artifacts'
 SFN_PATCH_CLASS = 'com/amazonaws/stepfunctions/local/runtime/executors/task/LambdaTaskStateExecutor.class'
 SFN_PATCH_CLASS_URL = '%s/raw/master/stepfunctions-local-patch/%s' % (ARTIFACTS_REPO, SFN_PATCH_CLASS)
 
-DEBUGPY_MODULE='debugpy'
-DEBUGPY_DEPENDENCIES=['gcc', 'python3-dev', 'musl-dev']
+# kinesis-mock version
+KINESIS_MOCK_VERSION = os.environ.get('KINESIS_MOCK_VERSION') or '0.1.0'
+KINESIS_MOCK_RELEASE_URL = 'https://api.github.com/repos/etspaceman/kinesis-mock/releases/tags/' + KINESIS_MOCK_VERSION
+
+DEBUGPY_MODULE = 'debugpy'
+DEBUGPY_DEPENDENCIES = ['gcc', 'python3-dev', 'musl-dev']
 
 # Target version for javac, to ensure compatibility with earlier JREs
 JAVAC_TARGET_VERSION = '1.8'
@@ -143,12 +152,69 @@ def install_elasticmq():
         shutil.copy(tmp_archive, INSTALL_DIR_ELASTICMQ)
 
 
+def install_kinesis():
+    if KINESIS_PROVIDER == 'kinesalite':
+        return install_kinesalite()
+    elif KINESIS_PROVIDER == 'kinesis-mock':
+        return install_kinesis_mock()
+    else:
+        raise ValueError('unknown kinesis provider %s' % KINESIS_PROVIDER)
+
+
 def install_kinesalite():
     if not os.path.exists(INSTALL_PATH_KINESALITE_CLI):
         log_install_msg('Kinesis')
         run('cd "%s" && npm install' % MODULE_MAIN_PATH)
 
 
+def install_kinesis_mock():
+    target_dir = INSTALL_PATH_KINESIS_MOCK
+
+    machine = platform.machine().lower()
+    system = platform.system().lower()
+    version = platform.version().lower()
+
+    is_probably_m1 = system == 'darwin' and ('arm64' in version or 'arm32' in version)
+
+    LOG.debug('getting kinesis-mock for %s %s', system, machine)
+    if ((machine == 'x86_64' or machine == 'amd64') and not is_probably_m1):
+        if system == 'windows':
+            bin_file = 'kinesis-mock-mostly-static.exe'
+        elif system == 'linux':
+            bin_file = 'kinesis-mock-linux-amd64-static'
+        elif system == 'darwin':
+            bin_file = 'kinesis-mock-macos-amd64-dynamic'
+        else:
+            bin_file = 'kinesis-mock.jar'
+    else:
+        bin_file = 'kinesis-mock.jar'
+
+    bin_file_path = os.path.join(target_dir, bin_file)
+    if os.path.exists(bin_file_path):
+        LOG.debug('kinesis-mock found at %s', bin_file_path)
+        return bin_file_path
+
+    response = requests.get(KINESIS_MOCK_RELEASE_URL)
+    if not response.ok:
+        raise ValueError('Could not get list of releases from %s: %s' % (KINESIS_MOCK_RELEASE_URL, response.text))
+
+    github_release = response.json()
+    download_url = None
+    for asset in github_release.get('assets', []):
+        # find the correct binary in the release
+        if asset['name'] == bin_file:
+            download_url = asset['browser_download_url']
+            break
+
+    if download_url is None:
+        raise ValueError('could not find required binary %s in release %s' % (bin_file, KINESIS_MOCK_RELEASE_URL))
+
+    mkdir(target_dir)
+    LOG.info('downloading kinesis-mock binary from %s', download_url)
+    download(download_url, bin_file_path)
+    return bin_file_path
+
+
 def install_local_kms():
     local_arch = get_arch()
     binary_path = INSTALL_PATH_KMS_BINARY_PATTERN.replace('<arch>', local_arch)
@@ -168,7 +234,7 @@ def install_stepfunctions_local():
         run('{dc} pull {img}'.format(dc=config.DOCKER_CMD, img=IMAGE_NAME_SFN_LOCAL))
         docker_name = 'tmp-ls-sfn'
         run(('{dc} run --name={dn} --entrypoint= -d --rm {img} sleep 15').format(
-                dc=config.DOCKER_CMD, dn=docker_name, img=IMAGE_NAME_SFN_LOCAL))
+            dc=config.DOCKER_CMD, dn=docker_name, img=IMAGE_NAME_SFN_LOCAL))
         time.sleep(5)
         run('{dc} cp {dn}:/home/stepfunctionslocal/ {tgt}'.format(dc=config.DOCKER_CMD,
             dn=docker_name, tgt=INSTALL_DIR_INFRA))
@@ -182,8 +248,6 @@ def install_stepfunctions_local():
         run(cmd)
 
 
-
-
 def install_dynamodb_local():
     if OVERWRITE_DDB_FILES_IN_DOCKER and in_docker():
         rm_rf(INSTALL_DIR_DDB)
@@ -249,7 +313,7 @@ def install_component(name):
     installers = {
         'cloudformation': install_cloudformation_libs,
         'dynamodb': install_dynamodb_local,
-        'kinesis': install_kinesalite,
+        'kinesis': install_kinesis,
         'kms': install_local_kms,
         'sqs': install_elasticmq,
         'stepfunctions': install_stepfunctions_local,
@@ -271,13 +335,15 @@ def install_all_components():
     # install all components
     install_components(DEFAULT_SERVICE_PORTS.keys())
 
+
 def install_debugpy_and_dependencies():
     try:
         import debugpy
+        assert debugpy
     except ModuleNotFoundError:
-        run('apk fetch %s --output %s'%  (' '.join(DEBUGPY_DEPENDENCIES), config.TMP_FOLDER))
-        run('apk add %s --cache-dir %s' %  (' '.join(DEBUGPY_DEPENDENCIES), config.TMP_FOLDER ) )
-        run('pip install %s' % DEBUGPY_MODULE )
+        run('apk fetch %s --output %s' % (' '.join(DEBUGPY_DEPENDENCIES), config.TMP_FOLDER))
+        run('apk add %s --cache-dir %s' % (' '.join(DEBUGPY_DEPENDENCIES), config.TMP_FOLDER))
+        run('pip install %s' % DEBUGPY_MODULE)
 
 
 # -----------------
@@ -322,8 +388,7 @@ def download_and_extract_with_retry(archive_url, tmp_archive, target_dir):
         download_and_extract(archive_url, target_dir, tmp_archive=tmp_archive)
 
 
-if __name__ == '__main__':
-
+def main():
     if len(sys.argv) > 1:
         os.environ['LOCALSTACK_API_KEY'] = os.environ.get('LOCALSTACK_API_KEY') or 'test'
         if sys.argv[1] == 'libs':
@@ -335,3 +400,7 @@ if __name__ == '__main__':
             # Install additional libraries for testing
             install_amazon_kinesis_client_libs()
         print('Done.')
+
+
+if __name__ == '__main__':
+    main()
