@@ -6,12 +6,14 @@ import codecs
 import random
 import logging
 import datetime
+
 import xmltodict
 import collections
 import dateutil.parser
 import urllib.parse
 import six
 import botocore.config
+from moto.s3.exceptions import InvalidFilterRuleName
 from pytz import timezone
 from urllib.parse import parse_qs
 from moto.s3.models import s3_backend
@@ -24,10 +26,10 @@ from localstack.services.s3 import multipart_content
 from localstack.services.s3.s3_utils import (
     is_static_website, extract_bucket_name, extract_key_name, validate_bucket_name, uses_host_addressing,
     SIGNATURE_V2_PARAMS, SIGNATURE_V4_PARAMS, authenticate_presign_url, get_forwarded_for_host,
-    ALLOWED_HEADER_OVERRIDES
+    ALLOWED_HEADER_OVERRIDES, is_expired
 )
 from localstack.utils.common import (
-    short_uid, timestamp_millis, to_str, to_bytes, clone, md5, get_service_protocol, now_utc, is_base64
+    short_uid, timestamp_millis, to_str, to_bytes, clone, md5, get_service_protocol, is_base64
 )
 from localstack.utils.analytics import event_publisher
 from localstack.utils.persistence import PersistingProxyListener
@@ -46,9 +48,6 @@ S3_NOTIFICATIONS = s3_backend.S3_NOTIFICATIONS = getattr(s3_backend, 'S3_NOTIFIC
 # mappings for bucket CORS settings
 BUCKET_CORS = s3_backend.BUCKET_CORS = getattr(s3_backend, 'BUCKET_CORS', {})
 
-# mappings for bucket payer settings
-BUCKET_PAYER = s3_backend.BUCKET_PAYER = getattr(s3_backend, 'BUCKET_PAYER', {})
-
 # maps bucket name to lifecycle settings
 BUCKET_LIFECYCLE = s3_backend.BUCKET_LIFECYCLE = getattr(s3_backend, 'BUCKET_LIFECYCLE', {})
 
@@ -412,6 +411,7 @@ def append_cors_headers(bucket_name, request_method, request_headers, response):
             allowed_origins = convert_origins_into_list(allowed_origins)
 
             for allowed in allowed_origins:
+                allowed = allowed or ''
                 if origin in allowed or re.match(allowed.replace('*', '.*'), origin):
 
                     response.headers['Access-Control-Allow-Origin'] = origin
@@ -466,12 +466,6 @@ def get_object_expiry(path):
     return OBJECT_EXPIRY.get(path)
 
 
-def is_url_already_expired(expiry_timestamp):
-    if int(expiry_timestamp) < int(now_utc()):
-        return True
-    return False
-
-
 def add_response_metadata_headers(response):
     if response.headers.get('content-language') is None:
         response.headers['content-language'] = 'en-US'
@@ -908,59 +902,115 @@ def normalize_bucket_name(bucket_name):
     return S3Bucket.normalize_bucket_name(bucket_name)
 
 
-def handle_notification_request(bucket, method, data):
+def empty_response():
     response = Response()
     response.status_code = 200
     response._content = ''
-    if method == 'GET':
-        # TODO check if bucket exists
-        result = '<NotificationConfiguration xmlns="%s">' % XMLNS_S3
-        if bucket in S3_NOTIFICATIONS:
-            notifs = S3_NOTIFICATIONS[bucket]
-            for notif in notifs:
-                for dest in NOTIFICATION_DESTINATION_TYPES:
-                    if dest in notif:
-                        dest_dict = {
-                            '%sConfiguration' % dest: {
-                                'Id': notif['Id'],
-                                dest: notif[dest],
-                                'Event': notif['Event'],
-                                'Filter': notif['Filter']
-                            }
-                        }
-                        result += xmltodict.unparse(dest_dict, full_document=False)
-        result += '</NotificationConfiguration>'
-        response._content = result
-
-    if method == 'PUT':
-        parsed = xmltodict.parse(data)
-        notif_config = parsed.get('NotificationConfiguration')
-        S3_NOTIFICATIONS[bucket] = []
-        for dest in NOTIFICATION_DESTINATION_TYPES:
-            config = notif_config.get('%sConfiguration' % (dest))
-            configs = config if isinstance(config, list) else [config] if config else []
-            for config in configs:
-                events = config.get('Event')
-                if isinstance(events, six.string_types):
-                    events = [events]
-                event_filter = config.get('Filter', {})
-                # make sure FilterRule is an array
-                s3_filter = _get_s3_filter(event_filter)
-                if s3_filter and not isinstance(s3_filter.get('FilterRule', []), list):
-                    s3_filter['FilterRule'] = [s3_filter['FilterRule']]
-                # create final details dict
-                notification_details = {
-                    'Id': config.get('Id', str(uuid.uuid4())),
-                    'Event': events,
-                    dest: config.get(dest),
-                    'Filter': event_filter
-                }
-                S3_NOTIFICATIONS[bucket].append(clone(notification_details))
     return response
 
 
+def handle_notification_request(bucket, method, data):
+    if method == 'GET':
+        return handle_get_bucket_notification(bucket)
+    if method == 'PUT':
+        return handle_put_bucket_notification(bucket, data)
+
+    return empty_response()
+
+
+def handle_get_bucket_notification(bucket):
+    response = Response()
+    response.status_code = 200
+    response._content = ''
+
+    # TODO check if bucket exists
+    result = '<NotificationConfiguration xmlns="%s">' % XMLNS_S3
+    if bucket in S3_NOTIFICATIONS:
+        notifs = S3_NOTIFICATIONS[bucket]
+        for notif in notifs:
+            for dest in NOTIFICATION_DESTINATION_TYPES:
+                if dest in notif:
+                    dest_dict = {
+                        '%sConfiguration' % dest: {
+                            'Id': notif['Id'],
+                            dest: notif[dest],
+                            'Event': notif['Event'],
+                            'Filter': notif['Filter']
+                        }
+                    }
+                    result += xmltodict.unparse(dest_dict, full_document=False)
+    result += '</NotificationConfiguration>'
+    response._content = result
+    return response
+
+
+def _validate_filter_rules(filter_doc):
+    rules = filter_doc.get('FilterRule')
+    if not rules:
+        return
+
+    for rule in rules:
+        name = rule.get('Name', '')
+        if name.lower() not in ['suffix', 'prefix']:
+            raise InvalidFilterRuleName(name)
+
+        # TODO: check what other rules there are
+
+
+def _sanitize_notification_filter_rules(filter_doc):
+    rules = filter_doc.get('FilterRule')
+    if not rules:
+        return
+
+    for rule in rules:
+        name = rule.get('Name', '')
+        if name.lower() not in ['suffix', 'prefix']:
+            raise InvalidFilterRuleName(name)
+
+        rule['Name'] = name.title()
+
+
+def handle_put_bucket_notification(bucket, data):
+    parsed = xmltodict.parse(data)
+    notif_config = parsed.get('NotificationConfiguration')
+
+    notifications = []
+
+    for dest in NOTIFICATION_DESTINATION_TYPES:
+        config = notif_config.get('%sConfiguration' % dest)
+        configs = config if isinstance(config, list) else [config] if config else []
+        for config in configs:
+            events = config.get('Event')
+            if isinstance(events, six.string_types):
+                events = [events]
+            event_filter = config.get('Filter', {})
+            # make sure FilterRule is an array
+            s3_filter = _get_s3_filter(event_filter)
+
+            if s3_filter and not isinstance(s3_filter.get('FilterRule', []), list):
+                s3_filter['FilterRule'] = [s3_filter['FilterRule']]
+
+            # make sure FilterRules are valid and sanitize if necessary
+            _sanitize_notification_filter_rules(s3_filter)
+
+            # create final details dict
+            notification_details = {
+                'Id': config.get('Id', str(uuid.uuid4())),
+                'Event': events,
+                dest: config.get(dest),
+                'Filter': event_filter
+            }
+
+            notifications.append(clone(notification_details))
+
+    S3_NOTIFICATIONS[bucket] = notifications
+
+    return empty_response()
+
+
 def remove_bucket_notification(bucket):
-    S3_NOTIFICATIONS.pop(bucket, None)
+    if bucket in S3_NOTIFICATIONS:
+        del S3_NOTIFICATIONS[bucket]
 
 
 def not_none_or(value, alternative):
@@ -1029,9 +1079,13 @@ class ProxyListenerS3(PersistingProxyListener):
     @staticmethod
     def parse_policy_expiration_date(expiration_string):
         try:
-            return datetime.datetime.strptime(expiration_string, POLICY_EXPIRATION_FORMAT1)
+            dt = datetime.datetime.strptime(expiration_string, POLICY_EXPIRATION_FORMAT1)
         except Exception:
-            return datetime.datetime.strptime(expiration_string, POLICY_EXPIRATION_FORMAT2)
+            dt = datetime.datetime.strptime(expiration_string, POLICY_EXPIRATION_FORMAT2)
+
+        # both date formats assume a UTC timezone ('Z' suffix), but it's not parsed as tzinfo into the datetime object
+        dt = dt.replace(tzinfo=datetime.timezone.utc)
+        return dt
 
     def forward_request(self, method, path, data, headers):
         # Create list of query parameteres from the url
@@ -1120,7 +1174,8 @@ class ProxyListenerS3(PersistingProxyListener):
 
         # if the Expires key in the url is already expired then return error
         if method == 'GET' and 'Expires' in query_map:
-            if is_url_already_expired(query_map.get('Expires')[0]):
+            ts = datetime.datetime.fromtimestamp(int(query_map.get('Expires')[0]), tz=datetime.timezone.utc)
+            if is_expired(ts):
                 return token_expired_error(path, headers.get('x-amz-request-id'), 400)
 
         # If multipart POST with policy in the params, return error if the policy has expired
@@ -1131,8 +1186,7 @@ class ProxyListenerS3(PersistingProxyListener):
                 expiration_string = policy.get('expiration', None)  # Example: 2020-06-05T13:37:12Z
                 if expiration_string:
                     expiration_datetime = self.parse_policy_expiration_date(expiration_string)
-                    expiration_timestamp = expiration_datetime.timestamp()
-                    if is_url_already_expired(expiration_timestamp):
+                    if is_expired(expiration_datetime):
                         return token_expired_error(path, headers.get('x-amz-request-id'), 400)
 
         if query == 'cors' or 'cors' in query_map:
@@ -1208,7 +1262,7 @@ class ProxyListenerS3(PersistingProxyListener):
 
             if response.status_code == 201 and key:
                 response._content = self.get_201_response(key, bucket_name)
-                response.headers['Content-Length'] = str(len(response._content))
+                response.headers['Content-Length'] = str(len(response._content or ''))
                 response.headers['Content-Type'] = 'application/xml; charset=utf-8'
                 return response
         if response.status_code == 416:
@@ -1347,7 +1401,7 @@ class ProxyListenerS3(PersistingProxyListener):
                 reset_content_length = True
 
             if reset_content_length:
-                response.headers['Content-Length'] = str(len(response._content))
+                response.headers['Content-Length'] = str(len(response._content or ''))
 
             # convert to chunked encoding, for compatibility with certain SDKs (e.g., AWS PHP SDK)
             convert_to_chunked_encoding(method, path, response)
