@@ -1,8 +1,9 @@
+import os
 import types
 import logging
 import traceback
-from moto.s3 import models as s3_models, responses as s3_responses, exceptions as s3_exceptions
-from moto.s3.responses import minidom, MalformedXML, undo_clean_key_name, is_delete_keys, S3_ALL_MULTIPARTS
+from moto.s3 import models as s3_models, responses as s3_responses
+from moto.s3.responses import minidom, MalformedXML, undo_clean_key_name, S3_ALL_MULTIPARTS, is_delete_keys
 from moto.s3.exceptions import S3ClientError
 from moto.s3bucket_path import utils as s3bucket_path_utils
 from localstack import config
@@ -12,13 +13,12 @@ from localstack.utils.server import multiserver
 from localstack.utils.common import wait_for_port_open, get_free_tcp_port
 from localstack.utils.generic.dict_utils import get_safe
 from localstack.services.infra import start_moto_server
-from localstack.services.awslambda.lambda_api import BUCKET_MARKER_LOCAL
 from urllib.parse import urlparse
 
 LOG = logging.getLogger(__name__)
 
-# max file size for S3 objects (in MB)
-S3_MAX_FILE_SIZE_MB = 2048
+# max file size for S3 objects kept in memory (500 KB by default)
+S3_MAX_FILE_SIZE_BYTES = 512 * 1024
 
 # temporary state
 TMP_STATE = {}
@@ -41,7 +41,7 @@ def check_s3(expect_shutdown=False, print_error=False):
     if expect_shutdown:
         assert out is None
     else:
-        assert isinstance(out['Buckets'], list)
+        assert out and isinstance(out.get('Buckets'), list)
 
 
 def start_s3(port=None, backend_port=None, asynchronous=None, update_listener=None):
@@ -67,7 +67,8 @@ def apply_patches():
 
     TMP_STATE[PATCHES_APPLIED] = True
 
-    s3_models.DEFAULT_KEY_BUFFER_SIZE = S3_MAX_FILE_SIZE_MB * 1024 * 1024
+    if not os.environ.get('MOTO_S3_DEFAULT_KEY_BUFFER_SIZE'):
+        os.environ['MOTO_S3_DEFAULT_KEY_BUFFER_SIZE'] = str(S3_MAX_FILE_SIZE_BYTES)
 
     def s3_update_acls(self, request, query, bucket_name, key_name):
         # fix for - https://github.com/localstack/localstack/issues/1733
@@ -109,7 +110,7 @@ def apply_patches():
     # patch S3Bucket.get_bucket(..)
     def get_bucket(self, bucket_name, *args, **kwargs):
         bucket_name = s3_listener.normalize_bucket_name(bucket_name)
-        if bucket_name == BUCKET_MARKER_LOCAL:
+        if bucket_name == config.BUCKET_MARKER_LOCAL:
             return None
         return get_bucket_orig(bucket_name, *args, **kwargs)
 
@@ -119,11 +120,8 @@ def apply_patches():
     # patch S3Bucket.get_bucket(..)
     def delete_bucket(self, bucket_name, *args, **kwargs):
         bucket_name = s3_listener.normalize_bucket_name(bucket_name)
-        try:
-            s3_listener.remove_bucket_notification(bucket_name)
-            return delete_bucket_orig(bucket_name, *args, **kwargs)
-        except s3_exceptions.MissingBucket:
-            pass
+        s3_listener.remove_bucket_notification(bucket_name)
+        return delete_bucket_orig(bucket_name, *args, **kwargs)
 
     delete_bucket_orig = s3_models.s3_backend.delete_bucket
     s3_models.s3_backend.delete_bucket = types.MethodType(delete_bucket, s3_models.s3_backend)
@@ -292,10 +290,11 @@ def apply_patches():
     # https://github.com/localstack/localstack/issues/2866
     # https://github.com/localstack/localstack/issues/2850
     # https://github.com/localstack/localstack/issues/3931
+    # https://github.com/localstack/localstack/issues/4015
     utils_is_delete_keys_orig = s3bucket_path_utils.is_delete_keys
 
     def utils_is_delete_keys(request, path, bucket_name):
-        return path == '/' + bucket_name + '?delete=' or utils_is_delete_keys_orig(request, path, bucket_name)
+        return '/' + bucket_name + '?delete=' in path or utils_is_delete_keys_orig(request, path, bucket_name)
 
     def s3_response_is_delete_keys(self, request, path, bucket_name):
         if self.subdomain_based_buckets(request):
