@@ -21,7 +21,7 @@ from localstack.services.cloudformation.service_models import (
     GenericBaseModel, DependencyNotYetSatisfied, KEY_RESOURCE_STATE)
 from localstack.services.cloudformation.deployment_utils import (
     dump_json_params, select_parameters, param_defaults, remove_none_values,
-    lambda_keys_to_lower, PLACEHOLDER_AWS_NO_VALUE, PLACEHOLDER_RESOURCE_NAME)
+    PLACEHOLDER_AWS_NO_VALUE, PLACEHOLDER_RESOURCE_NAME)
 
 ACTION_CREATE = 'create'
 ACTION_DELETE = 'delete'
@@ -252,12 +252,6 @@ RESOURCE_TO_FUNCTION = {
     },
     'ApiGateway::Account': {
     },
-    'ApiGateway::Stage': {
-        'create': {
-            'function': 'create_stage',
-            'parameters': lambda_keys_to_lower()
-        }
-    },
     'ApiGateway::Model': {
         'create': {
             'function': 'create_model',
@@ -429,6 +423,8 @@ def get_service_name(resource):
         return 'es'
     if parts[-2] == 'KinesisFirehose':
         return 'firehose'
+    if parts[-2] == 'ResourceGroups':
+        return 'resource-groups'
     return parts[1].lower()
 
 
@@ -547,6 +543,9 @@ def extract_resource_attribute(resource_type, resource_state, attribute, resourc
     if isinstance(resource_state, MotoCloudFormationModel):
         if is_ref_attribute:
             res_phys_id = getattr(resource_state, 'physical_resource_id', None)
+            get_res_phys_id = getattr(resource_state, 'get_physical_resource_id', None)
+            if not res_phys_id and get_res_phys_id:
+                res_phys_id = get_res_phys_id(attribute)
             if res_phys_id:
                 return res_phys_id
         if hasattr(resource_state, 'get_cfn_attribute'):
@@ -779,7 +778,12 @@ def resolve_refs_recursively(stack_name, value, resources):
         if stripped_fn_lower == 'importvalue':
             import_value_key = resolve_refs_recursively(stack_name, value[keys_list[0]], resources)
             stack = find_stack(stack_name)
-            return stack.exports_map[import_value_key]['Value']
+            stack_export = stack.exports_map.get(import_value_key) or {}
+            if not stack_export.get('Value'):
+                LOG.info('Unable to find export "%s" in stack "%s", existing export names: %s' %
+                    (import_value_key, stack_name, list(stack.exports_map.keys())))
+                return None
+            return stack_export['Value']
 
         if stripped_fn_lower == 'if':
             condition, option1, option2 = value[keys_list[0]]
@@ -1239,6 +1243,9 @@ def run_post_create_actions(action_name, resource_id, resources, resource_type,
             if integration.get('RequestTemplates'):
                 kwargs['requestTemplates'] = integration['RequestTemplates']
 
+            if integration.get('Credentials'):
+                kwargs['credentials'] = integration['Credentials']
+
             apigateway.put_integration(
                 restApiId=api_id,
                 resourceId=res_id,
@@ -1282,8 +1289,15 @@ def run_post_create_actions(action_name, resource_id, resources, resource_type,
     elif resource_type == 'IAM::Role':
         policies = resource_props.get('Policies', [])
         for policy in policies:
+            policy = policy[0] if isinstance(policy, list) and len(policy) == 1 else policy
             iam = aws_stack.connect_to_service('iam')
-            pol_name = policy['PolicyName']
+            if policy == PLACEHOLDER_AWS_NO_VALUE:
+                continue
+            if not isinstance(policy, dict):
+                LOG.info('Invalid format of policy for IAM role "%s": %s' % (
+                    resource_props.get('RoleName'), policy))
+                continue
+            pol_name = policy.get('PolicyName')
             doc = dict(policy['PolicyDocument'])
             doc['Version'] = doc.get('Version') or IAM_POLICY_VERSION
             statements = doc['Statement'] if isinstance(doc['Statement'], list) else [doc['Statement']]
