@@ -15,11 +15,11 @@ import logging
 import tarfile
 import zipfile
 import binascii
-import calendar
 import tempfile
 import functools
 import threading
 import subprocess
+
 import six
 import shutil
 import requests
@@ -39,6 +39,26 @@ from localstack.config import DEFAULT_ENCODING
 from localstack.constants import ENV_DEV
 from localstack.utils.bootstrap import FuncThread
 
+# this is very ugly, but necessary to make timezone work with python 2.7 (which is used in lambdas)
+try:
+    from datetime import timezone
+
+    utc = timezone.utc
+except ImportError:
+    from datetime import tzinfo, timedelta
+
+    class UTC(tzinfo):
+        def utcoffset(self, dt):
+            return timedelta(0)
+
+        def tzname(self, dt):
+            return 'UTC'
+
+        def dst(self, dt):
+            return timedelta(0)
+
+    utc = UTC()
+
 # arrays for temporary files and resources
 TMP_FILES = []
 TMP_THREADS = []
@@ -339,6 +359,7 @@ class CaptureOutput(object):
 # ----------------
 
 def start_thread(method, *args, **kwargs):
+    """ Start the given method in a background thread, and add the thread to the TMP_THREADS shutdown hook """
     _shutdown_hook = kwargs.pop('_shutdown_hook', True)
     thread = FuncThread(method, *args, **kwargs)
     thread.start()
@@ -681,9 +702,15 @@ def recurse_object(obj, func, path=''):
     return obj
 
 
-def keys_to_lower(obj):
-    """ Recursively changes all dict keys to first character lowercase. """
-    def fix_keys(o, **kwargs):
+def keys_to_lower(obj, skip_children_of=None):
+    """ Recursively changes all dict keys to first character lowercase. Skip children
+        of any elements whose names are contained in skip_children_of (e.g., ['Tags']) """
+    skip_children_of = skip_children_of or []
+    skip_children_of = skip_children_of if isinstance(skip_children_of, list) else [skip_children_of]
+
+    def fix_keys(o, path='', **kwargs):
+        if any([re.match(r'(^|.*\.)%s($|[.\[].*)' % k, path) for k in skip_children_of]):
+            return o
         if isinstance(o, dict):
             for k, v in dict(o).items():
                 o.pop(k)
@@ -719,19 +746,18 @@ def obj_to_xml(obj):
     return str(obj)
 
 
+def now(millis=False, tz=None):
+    return mktime(datetime.now(tz=tz), millis=millis)
+
+
 def now_utc(millis=False):
-    return mktime(datetime.utcnow(), millis=millis)
+    return now(millis, utc)
 
 
-def now(millis=False):
-    return mktime(datetime.now(), millis=millis)
-
-
-def mktime(timestamp, millis=False):
+def mktime(ts, millis=False):
     if millis:
-        epoch = datetime.utcfromtimestamp(0)
-        return (timestamp - epoch).total_seconds()
-    return calendar.timegm(timestamp.timetuple())
+        return ts.timestamp() * 1000
+    return ts.timestamp()
 
 
 def mkdir(folder):
@@ -976,6 +1002,20 @@ def long_uid():
     return str(uuid.uuid4())
 
 
+def parse_json_or_yaml(markup):
+    import yaml  # leave import here, to avoid breaking our Lambda tests!
+    try:
+        return json.loads(markup)
+    except Exception:
+        try:
+            return clone_safe(yaml.safe_load(markup))
+        except Exception:
+            try:
+                return clone_safe(yaml.load(markup, Loader=yaml.SafeLoader))
+            except Exception:
+                raise
+
+
 def json_safe(item):
     """ return a copy of the given object (e.g., dict) that is safe for JSON dumping """
     try:
@@ -1052,6 +1092,14 @@ def get_or_create_file(file_path, content=None):
         pass
 
 
+def replace_in_file(search, replace, file_path):
+    """ Replace all occurrences of `search` with `replace` in the given file (overwrites in place!) """
+    content = load_file(file_path) or ''
+    content_new = content.replace(search, replace)
+    if content != content_new:
+        save_file(file_path, content_new)
+
+
 def to_str(obj, encoding=DEFAULT_ENCODING, errors='strict'):
     """ If ``obj`` is an instance of ``binary_type``, return
     ``obj.decode(encoding, errors)``, otherwise return ``obj`` """
@@ -1075,6 +1123,16 @@ def str_remove(string, index, end_index=None):
     return '%s%s' % (string[:index], string[end_index:])
 
 
+def last_index_of(array, value):
+    """ Return the last index of `value` in the given list, or -1 if it does not exist. """
+    result = -1
+    for i in reversed(range(len(array))):
+        entry = array[i]
+        if entry == value or (callable(value) and value(entry)):
+            return i
+    return result
+
+
 def is_sub_dict(child_dict, parent_dict):
     """ Returns whether the first dict is a sub-dict (subset) of the second dict. """
     return all(parent_dict.get(key) == val for key, val in child_dict.items())
@@ -1085,11 +1143,11 @@ def cleanup(files=True, env=ENV_DEV, quiet=True):
         cleanup_tmp_files()
 
 
-def cleanup_threads_and_processes(quiet=True, debug=False):
+def cleanup_threads_and_processes(quiet=True):
     for thread in TMP_THREADS:
         if thread:
             try:
-                print_debug('[shutdown] Cleaning up thread: %s' % thread, debug)
+                # LOG.debug('[shutdown] Cleaning up thread: %s', thread)
                 if hasattr(thread, 'shutdown'):
                     thread.shutdown()
                     continue
@@ -1101,7 +1159,7 @@ def cleanup_threads_and_processes(quiet=True, debug=False):
                 print(e)
     for proc in TMP_PROCESSES:
         try:
-            print_debug('[shutdown] Cleaning up process: %s' % proc, debug)
+            # LOG.debug('[shutdown] Cleaning up process: %s', proc)
             kill_process_tree(proc.pid)
             # proc.terminate()
         except Exception as e:
@@ -1111,16 +1169,16 @@ def cleanup_threads_and_processes(quiet=True, debug=False):
         import asyncio
         for task in asyncio.all_tasks():
             try:
-                print_debug('[shutdown] Canceling asyncio task: %s' % task, debug)
+                # LOG.debug('[shutdown] Canceling asyncio task: %s', task)
                 task.cancel()
             except Exception as e:
                 print(e)
     except Exception:
         pass
-    print_debug('[shutdown] Done cleaning up threads / processes / tasks', debug)
+    LOG.debug('[shutdown] Done cleaning up threads / processes / tasks')
     # clear lists
-    clear_list(TMP_THREADS)
-    clear_list(TMP_PROCESSES)
+    TMP_THREADS.clear()
+    TMP_PROCESSES.clear()
 
 
 def kill_process_tree(parent_pid):
@@ -1136,11 +1194,6 @@ def kill_process_tree(parent_pid):
     parent.kill()
 
 
-def clear_list(list_obj):
-    while len(list_obj):
-        del list_obj[0]
-
-
 def items_equivalent(list1, list2, comparator):
     """ Returns whether two lists are equivalent (i.e., same items contained in both lists,
         irresepective of the items' order) with respect to a comparator function. """
@@ -1275,14 +1328,9 @@ def is_root():
     return out == 'root'
 
 
-def cleanup_resources(debug=False):
+def cleanup_resources():
     cleanup_tmp_files()
-    cleanup_threads_and_processes(debug=debug)
-
-
-def print_debug(msg, debug=False):
-    if debug:
-        print(msg)
+    cleanup_threads_and_processes()
 
 
 @synchronized(lock=SSL_CERT_LOCK)
